(*
    FCE
    Copyright (c) 1996 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
{$I-}
unit Database;

interface

uses
  Chord,
  Forms, SysUtils, WinTypes, WinProcs, StdCtrls;

function NewChord: PChord;
function LoadChords( LB: TListBox ): integer;
function SaveChords( LB: TListBox ): integer;

implementation

const
  CHORD_PER_CHUNK = 256;
  MAX_ALLOC_CHUNK = 64;
  MAX_CHORD = MAX_ALLOC_CHUNK * CHORD_PER_CHUNK;
var
  Chunk: array[ 0..MAX_ALLOC_CHUNK-1 ] of PByte;
  ChordIndex: integer;
  FileName: string;

(* A chord allocation function to avoid *)
(* memory fragmentation *)
function NewChord: PChord;
var
  C, I: integer;
  P   : PByte;
begin
  C := ChordIndex div CHORD_PER_CHUNK;
  I := ChordIndex mod CHORD_PER_CHUNK;
  if( Chunk[C] = nil ) then
    GetMem( Chunk[C], CHORD_PER_CHUNK * sizeof(TChord) );
  P := Chunk[C];
  Inc( P, I*sizeof(TChord) );
  NewChord := PChord(P);
  Inc( ChordIndex );
end;

procedure FreeAllChords;
var
  I: integer;
begin
  for I:=0 to MAX_ALLOC_CHUNK-1 do begin
    if( Chunk[I] <> nil ) then
      FreeMem( Chunk[I], CHORD_PER_CHUNK*sizeof(TChord) );
  end;
end;

function LoadChords( LB: TListBox ): integer;
var
  F: file;
  P: PChord;
begin
  LoadChords := -1;
  LB.Clear;
  Assign( F, FileName );
  Reset( F, 1 );
  if( IOResult = 0 ) then begin
    while not( Eof(F) ) do begin
      P := NewChord;
      BlockRead( F, P^, sizeof(TChord) );
      LB.Items.AddObject( P^.Name, TObject(P) );
    end;
  end;
  Close( F );
  LoadChords := 0;
end;

function SaveChords( LB: TListBox ): integer;
var
  F: file;
  I: integer;
begin
  SaveChords := -1;
  Assign( F, FileName );
  ReWrite( F, 1 );
  if( IoResult = 0 ) then begin
    for I:=1 to LB.Items.Count do begin
      BlockWrite( F, PChord(LB.Items.Objects[I-1])^, sizeof(TChord) );
    end;
  end;
  Close( F );
  SaveChords := 0;
end;

var
  I: integer;
initialization
  ChordIndex := 0;
  for I:=0 to MAX_ALLOC_CHUNK-1 do
    Chunk[I] := nil;
  FileName := ChangeFileExt( Application.ExeName, '.CDB' );
end.
