(*
    FCE
    Copyright (c) 1996 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
unit Find;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls;

type
  TfrmFindWnd = class(TForm)
    Label2: TLabel;
    Panel1: TPanel;
    imgWindow: TImage;
    optHide: TCheckBox;
    btnOk: TBitBtn;
    btnCancel: TBitBtn;
    GroupBox1: TGroupBox;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    lblClass: TLabel;
    lblHandle: TLabel;
    lblTitle: TLabel;
    optParent: TCheckBox;
    procedure FormActivate(Sender: TObject);
    procedure imgWindowMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure imgWindowMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure imgWindowMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    { Private declarations }
    boTracking: boolean;
    procedure SetTargetWindow( H: HWnd );
  public
    { Public declarations }
    hwndTarget: HWnd;
  end;

var
  frmFindWnd: TfrmFindWnd;

implementation

uses
  Extra,
  Main;

{$R *.DFM}

procedure TfrmFindWnd.FormActivate(Sender: TObject);
begin
  boTracking := FALSE;
  SetTargetWindow( 0 );
end;

procedure TfrmFindWnd.SetTargetWindow( H: HWnd );
var
  S: string;
  P: HWnd;
begin
  if( hwndTarget <> H )or( H = 0 ) then begin
    (* Find parent window *)
    P := H;
    if( P <> 0 ) then
      while( GetParent(P) <> 0 ) do
        P := GetParent( P );
    (* Do not show our own window *)
    if( P = Self.Handle ) then
      H := 0;
    (* Show information *)
    if( H = 0 ) then begin
      hwndTarget := H;
      lblHandle.Caption := '(none)';
      lblTitle.Caption := '';
      lblClass.Caption := '';
    end
    else begin
      if( optParent.Checked ) then
        H := P;
      hwndTarget := H;
      lblHandle.Caption := IntToHex( H, 4 ) + 'h';
      S[0] := char( GetWindowText( H, @S[1], 255 ) );
      lblTitle.Caption := S;
      S[0] := char( GetClassName( H, @S[1], 255 ) );
      lblClass.Caption := S;
    end;
  end;
end;

procedure TfrmFindWnd.imgWindowMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  boTracking := TRUE;
  SetTargetWindow( 0 );
  if( optHide.Checked ) then
    frmMain.Hide;
  imgWindow.Hide;
  Screen.Cursor := CUR_WINDOW;
end;

procedure TfrmFindWnd.imgWindowMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  boTracking := FALSE;
  if( optHide.Checked ) then begin
    frmMain.Show;
    Self.Show;
  end;
  Screen.Cursor := crDefault;
  imgWindow.Show;
end;

procedure TfrmFindWnd.imgWindowMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var
  P: TPoint;
  H: HWnd;
begin
  if( boTracking ) then begin
    GetCursorPos( P );
    H := WindowFromPoint( P );
    SetTargetWindow( H );
  end;
end;

end.
